"""\
Author: Moshe Zadka <mzadka@geocities.com>

Copyright (c) Moshe Zadka, 1999
All rights reserved.

redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  o Redistributions of source code must retain the above copyright
    notice, this list of conditions, and the disclaimer that follows.

  o Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions, and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  o Redistributions of source code must retain the above copyright
    notice, this list of conditions, and the disclaimer that follows.

  o Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions, and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

  o The name of the contributor may not be used to endorse or promote 
    products derived from this software without specific prior written 
    permission.


THIS SOFTWARE IS PROVIDED BY *AS IS* AND ANY EXPRESS OR IMPLIED WARRANTIES, 
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
AUTHOR CREATIONS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.

exparse: parse ``explanation file''

On my system, I install things in /usr/local, and I want to remember
what everything is. exparse is a small utility helping me.

It analyzes a file called ``explanation'' whose format is something
like (example):

        Vim--A VI emulator. Version 5.3 with python support is installed
        /usr/local/share/vim/....
        /usr/local/bin/vim
        .
        .
        Gendoc--A utility to generate documents from doc-strings.
        /usr/local/lib/python1.5/site-packages/gendoc/__init__.py
        .
        .

There are also utilities to check all the files in /usr/local, to see
all are accounted for. Of course, everything should really be in global
variables, but that is still in the future.

To use this module as a program run it with arguments
-f <filename>.

It will analyze the file, tell you how many files are unaccounted for,
and will put them in that file under the heading of "Undocumented", so
you will be able to easily edit the file and document the entries.

The default filename is "explanation".
"""
import string, operator, posixpath, sys

class parser:
        """\
        A class to parse an explanation file.

        See module docstring for documentation of the format.

        The class behaves as a dictionary. The keys are the parts
        before the ``--'', and the values are pairs: the first element
        is the part after the ``--'', and the second argument is the list
        of entries under that heading.

        The dictionary is regarded as immutable: there is currently
        no way to change the file by changing the dictionary, so there
        is no __setitem__ method.

        Other then the dictionary methods, the class has a method
        ``knownfiles'', which are all the known entries.
        """
        def __init__(self, file):
                """\
                construct an object.

                Arguments:

                'file' -- an open file-like object (must have readline())
                """
                self.file=file
                self.exp={}
                self._parse()
                self.file.close()
                self.file=None

        def _parse(self):
                while 1:
                        line=self.file.readline()
                        if not line: return
                        line=line[:-1]
                        if line[0]!='/':
                                name, description=string.split(line, '--', 1)
                                self.current=name
                                self.exp[name]=(description, [])
                        else:
                                self.exp[self.current][1].append(line)

        def __getitem__(self, k):
                """return the value under heading k"""
                return self.exp[k]

        def keys(self):
                """return list of headings"""
                return self.exp.keys()

        def has_key(self, k):
                """check if we know of a heading k"""
                return self.exp.has_key(k)

        def values(self):
                """return the list of all values"""
                return self.exp.values()

        def items(self):
                """return a list of (k, self[k]) where k in self.keys()"""
                return self.exp.items()

        def knownfiles(self):
                """return a list of all entries accounted for"""
                return reduce(operator.add,
                              map(lambda x: x[1], self.values()))

def files_in(path):
        """Recursive listing of path"""
        def func(to_add, dir, names):
                k=len(to_add)
                to_add[k:k]=map(operator.add, [dir+'/']*len(names), names)
        files=[]
        posixpath.walk(path, func, files)
        return files

def unknown_files(p):
        """return files in /usr/local p (a parser) does not know about"""
        all=files_in('/usr/local')
        known=p.knownfiles()
        for n in known:
                all.remove(n)
        for n in all[:]:
                if posixpath.isdir(n): all.remove(n)
        return all

def _test(file):
        """main program: analyze file"""
        p=parser(open(file))
        unknown=unknown_files(p)
        if unknown:
                f=open(file, "a")
                f.write("Undocumented--Files nothing was said about\n")
                for n in unknown:
                        f.write(n+'\n')
                f.close()
                print "There are %d new files to log" % len(unknown)
        else:
                print "Good job. All files are properly logged."

if __name__=="__main__":
        filename="explanation"
        files=filter(lambda x: x[1][:2]=='-f',
                     map(None, range(len(sys.argv[1:])), sys.argv[1:]))
        if files:
                file=files[-1]
                if file[-1][2:]: filename=file[2:]
                else: filename=file[sys.argv[file[0]]]
        _test(filename)
